unit Spindb;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, Spin, DBTables, DB;

type
  TDBSpinEdit = class(TSpinEdit)
  private
    FMinValue: longint;
    FMaxValue: longint;
    FIncrement: longint;
    FReadOnly: boolean;
    FDataLink: TFieldDataLink;
    function GetDataField: string;
    function GetDataSource: TDataSource;
    procedure SetDataField(const VDataField: string);
    procedure SetDataSource(VDataSource: TDataSource);
    procedure DataChange(Sender: TObject);
    procedure SetValue(NewValue: longint);
    function CheckValue(NewValue: longint): longint;
    procedure CMEnter(var Message: TCMGotFocus); message CM_ENTER;
    procedure CMExit(var Message: TCMExit); message CM_EXIT;
    procedure UpdateData(Sender: TObject);

  protected
    procedure DownClick(Sender: TObject); override;
    procedure UpClick(Sender: TObject); override;
    function IsValidChar(Key: Char): boolean; override;

  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

  published
    property ReadOnly: boolean read FReadOnly write FReadOnly;
    property DataField: string read GetDataField write SetDataField;
    property DataSource: TDataSource read GetDataSource write SetDataSource;
  end;

procedure Register;

implementation

constructor TDBSpinEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FIncrement := 1;
  FDataLink := TFieldDataLink.Create;
  FDataLink.OnDataChange := DataChange;
  FDataLink.OnUpdateData := UpdateData;
end;

destructor TDBSpinEdit.Destroy;
begin
  FDataLink.OnDataChange := nil;
  FDataLink.Free;
  inherited Destroy;
end;

procedure Register;
begin
  RegisterComponents('Data Controls', [TDBSpinEdit]);
end;

function TDBSpinEdit.GetDataField: string;
begin
  Result := FDataLink.FieldName;
end;

function TDBSpinEdit.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

procedure TDBSpinEdit.SetDatafield(const VDatafield: string);
begin
  FDataLink.FieldName := VDatafield;
end;

procedure TDBSpinEdit.SetDataSource(VDataSource: TDataSource);
begin
  FDataLink.DataSource := VDataSource;
end;

procedure TDBSpinEdit.DataChange(Sender: TObject);
begin
  if FDataLink.Field = nil then Value := 0
  else Value := FDataLink.Field.AsInteger;
end;

procedure TDBSpinEdit.SetValue(NewValue: longint);
begin
  Text := IntToStr(CheckValue(NewValue));
  UpdateData(Self);
end;

function TDBSpinEdit.CheckValue(NewValue: longint): longint;
begin
  Result := NewValue;
  if (FMaxValue <> FMinValue) then
    begin
    if NewValue < FMinValue then Result := FMinValue
    else if NewValue > FMaxValue then Result := FMaxvalue;
    end;
end;

procedure TDBSpinEdit.CMEnter(var Message: TCMGotFocus);
begin
  inherited;
  UpdateData(Self);
end;

procedure TDBSpinEdit.CMExit(var Message: TCMExit);
begin
  try
    UpdateData(Self);
  except
    SetFocus;
    raise;
  end;
  inherited;
end;

procedure TDBSpinEdit.UpdateData(Sender: TObject);
begin
  FDataLink.Modified;
  FDataLink.Edit;
  FDataLink.Field.AsInteger := Value;
end;

procedure TDBSpinEdit.DownClick(Sender: TObject);
begin
  if ReadOnly then MessageBeep(0)
  else begin
    Value := Value - FIncrement;
    UpdateData(Self);
    end;
end;

procedure TDBSpinEdit.UpClick(Sender: TObject);
begin
  if ReadOnly then MessageBeep(0)
  else begin
    Value := Value + FIncrement;
    UpdateData(Self);
    end;
end;

function TDBSpinEdit.IsValidChar(Key: Char): boolean;
begin
  if ReadOnly then MessageBeep(0)
  else inherited IsValidChar(Key);
  if Result then UpdateData(Self);
end;

end.
